Attribute VB_Name = "Window_Misc"
Option Explicit

'==================================================================
'
' Window Misc Properties
'
' 30 September 2001
'
' You're free in using this software, as long as the
' orignal copyright/author information remains intact.
' In other words, respecting the person(s) who made it
' possible for you to view/learn something of this software.
'
' Copyright  2001 - Diederik van der Boor
' http://www.codingdomain.com
'
'==================================================================


'------------------------------------------------------------------
' Public Constants

Public Enum WindowFlashFlags
  FLASHW_STOP = 0                               ' Stop flashing. The system restores the window to its original state.
  FLASHW_CAPTION = &H1                          ' Flash the window caption.
  FLASHW_TRAY = &H2                             ' Flash the taskbar button.
  FLASHW_ALL = (FLASHW_CAPTION Or FLASHW_TRAY)  ' Flash both the window caption and taskbar button. This is equivalent to setting the FLASHW_CAPTION Or FLASHW_TRAY flags.
  FLASHW_TIMER = &H4                            ' Flash continuously, until the FLASHW_STOP flag is set.
  FLASHW_TIMERNOFG = &HC                        ' Flash continuously, until the window comes to the foreground.
End Enum

Public Enum WindowState
  SW_NORMAL = 1
  SW_MAXIMIZE = 3
  SW_MINIMIZE = 6
  SW_RESTORE = 9
End Enum


'------------------------------------------------------------------
' Structures and API's

Private Type FLASHWINFO
  cbSize As Long
  hWnd As Long
  dwFlags As Long
  uCount As Long
  dwTimeout As Long
End Type

Private Enum API_SetWindowPos_Flags
  SWP_FRAMECHANGED = &H20
  SWP_HIDEWINDOW = &H80
  SWP_NOACTIVATE = &H10
  SWP_NOCOPYBITS = &H100
  SWP_NOMOVE = &H2
  SWP_NOOWNERZORDER = &H200
  SWP_NOREDRAW = &H8
  SWP_NOSIZE = &H1
  SWP_NOZORDER = &H4
  SWP_SHOWWINDOW = &H40
End Enum

Private Enum GetClassLong
  GCL_CBCLSEXTRA = (-20)
  GCL_CBWNDEXTRA = (-18)
  GCL_CONVERSION = &H1
  GCL_HBRBACKGROUND = (-10)
  GCL_HCURSOR = (-12)
  GCL_HICON = (-14)
  GCL_HMODULE = (-16)
  GCL_MENUNAME = (-8)
  GCL_REVERSE_LENGTH = &H3
  GCL_REVERSECONVERSION = &H2
  GCL_STYLE = (-26)
  GCL_WNDPROC = (-24)
End Enum

Private Declare Function GetClassLong Lib "user32" Alias "GetClassLongA" (ByVal hWnd As Long, ByVal nIndex As GetClassLong) As Long
Private Declare Function SetClassLong Lib "user32" Alias "SetClassLongA" (ByVal hWnd As Long, ByVal nIndex As GetClassLong, ByVal dwNewLong As Long) As Long
Private Declare Function SetWindowPos Lib "user32" (ByVal hWnd As Long, ByVal hWndInsertAfter As Long, ByVal X As Long, ByVal Y As Long, ByVal cx As Long, ByVal cy As Long, ByVal wFlags As API_SetWindowPos_Flags) As Long
Private Declare Function ShowWindow Lib "user32" (ByVal hWnd As Long, ByVal nCmdShow As WindowState) As Long
Private Declare Function GetWindowThreadProcessId Lib "user32" (ByVal hWnd As Long, lpdwProcessId As Long) As Long
Private Declare Function GetCurrentProcessId Lib "kernel32" () As Long
Private Declare Function LockWindowUpdate Lib "user32" (ByVal hWndLock As Long) As Long
Private Declare Function IsWindow Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function FlashWindowEx Lib "user32" (pfwi As FLASHWINFO) As Long
Private Declare Function EnableWindow Lib "user32" (ByVal hWnd As Long, ByVal fEnable As Long) As Long
Private Declare Function IsWindowEnabled Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function IsWindowVisible Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function GetClassName Lib "user32" Alias "GetClassNameA" (ByVal hWnd As Long, ByVal lpClassName As String, ByVal nMaxCount As Long) As Long


'------------------------------------------------------------------
' Window Enabled?

Public Property Get WindowEnabled(hWnd As OLE_HANDLE) As Boolean
  If IsWindow(hWnd) = 0 Then Error 5
  WindowEnabled = IsWindowEnabled(hWnd)
End Property

Public Property Let WindowEnabled(hWnd As OLE_HANDLE, NewValue As Boolean)
  If IsWindow(hWnd) = 0 Then Error 5
  If EnableWindow(hWnd, IIf(NewValue, 1, 0)) = 0 Then _
    Err.Raise 12345, "WindowEnabled", "Could not enable/disable window!"
End Property


'------------------------------------------------------------------
' Window Visible?

Public Property Get WindowVisible(hWnd As OLE_HANDLE) As Boolean
  If IsWindow(hWnd) = 0 Then Error 5
  WindowVisible = IsWindowVisible(hWnd)
End Property

Public Property Let WindowVisible(hWnd As OLE_HANDLE, NewValue As Boolean)
  If IsWindow(hWnd) = 0 Then Error 5
  If SetWindowPos(hWnd, 0, 0, 0, 0, 0, SWP_NOACTIVATE Or SWP_NOMOVE Or SWP_NOSIZE Or IIf(NewValue, SWP_SHOWWINDOW, SWP_HIDEWINDOW)) = 0 Then _
    Err.Raise 12345, "WindowVisible", "Could not show/hide window!"
End Property


'------------------------------------------------------------------
' Lock for update of drawing

Public Sub WindowLockedForUpdate(hWnd As OLE_HANDLE)
  If IsWindow(hWnd) = 0 And hWnd <> 0 Then Error 5
  If LockWindowUpdate(hWnd) = 0 Then _
    Err.Raise 12345, "WindowLockedForUpdate", "Could not lock window for update!"
End Sub


'------------------------------------------------------------------
' Flash the title bar

Public Sub WindowCaptionFlash(hWnd As OLE_HANDLE, Times As Long, TimeOut As Long, Flags As WindowFlashFlags)
  Dim FlashInfo As FLASHWINFO
  
  If IsWindow(hWnd) = 0 Then Error 5
  FlashInfo.cbSize = Len(FlashInfo)
  FlashInfo.dwFlags = Flags
  FlashInfo.dwTimeout = TimeOut
  FlashInfo.hWnd = hWnd
  FlashInfo.uCount = Times
  
  If FlashWindowEx(FlashInfo) = 0 Then _
    Err.Raise 12345, "WindowCaptionFlash", "Could not flash window!"
End Sub


'------------------------------------------------------------------
' Window state

Public Property Let WindowState(hWnd As OLE_HANDLE, NewValue As WindowState)
  If IsWindow(hWnd) = 0 Then Error 5
  If ShowWindow(hWnd, NewValue) = 0 Then _
    Err.Raise 12345, "WindowState", "Could not set window state!"
End Property


'------------------------------------------------------------------
' Process ID

Public Property Get WindowProcessId(hWnd As OLE_HANDLE) As Long
  Dim ID As Long
  If IsWindow(hWnd) = 0 Then Error 5
  If GetWindowThreadProcessId(hWnd, ID) = 0 Then _
    Err.Raise 12345, "WindowProcessID", "Could not get window process thread id!"
  WindowProcessId = ID
End Property

Public Function WindowIsLocal(hWnd As OLE_HANDLE)
  WindowIsLocal = WindowProcessId(hWnd) = GetCurrentProcessId()
End Function


'------------------------------------------------------------------
' Class name

Public Property Get WindowClassName(hWnd As OLE_HANDLE) As String
  Dim ClsName As String
  Dim ClsLen As String
  If IsWindow(hWnd) = 0 Then Error 5
  ClsLen = 128
  ClsName = VBA.Space(ClsLen)
  ClsLen = GetClassName(hWnd, ClsName, ClsLen)
  WindowClassName = Left(ClsName, ClsLen)
End Property
